import logging

from hopara.filter import Filter
from hopara.hopara import Hopara
from hopara.table import Table
from hopara.column_type import ColumnType

logging.basicConfig(level=logging.INFO, format='[%(asctime)s - %(levelname)s] %(message)s')
logging.getLogger(__name__).addHandler(logging.NullHandler())



from typing import Union
from hopara.column_type import ColumnType


class Table:
    """Hopara Table type.
    """
    def __init__(self, name: str):
        """Initialize a table with a name.
        :param name: name of the table.
        :type name: str
        """
        self.name = name
        self.__table = {'createStats': False}
        self.__columns = {}

    def get_payload(self) -> dict:
        self.__table['columns'] = [dict(properties, **{'name': name}) for name, properties in self.__columns.items()]
        return self.__table

    def set_create_stats(self, create_stats: bool):
        self.__table['createStats'] = create_stats

    def add_column(self, column_name: str, column_type: ColumnType):
        """Add a new columns to the table.
        :param column_name: type of the column
        :type column_name: hopara.ColumnType
        :param column_type: type of the column. See more details about these types at hopara.ColumnType.
        :type column_type: str
        """
        self.__columns[column_name] = {}
        if column_type:
            self.__columns[column_name]['type'] = column_type.name
        if column_name.startswith('_'):
            self.__columns[column_name]['label'] = column_name.lstrip('_')

    def __update_column_properties(self, column_name: Union[str, list], properties: dict):
        if isinstance(column_name, str):
            self.__columns[column_name].update(properties)
        else:
            for col_name in column_name:
                self.__columns[col_name].update(properties)

    def set_columns_type(self, column_name: Union[str, list], column_type: ColumnType):
        """Change a previously set or missing column type.
        :param column_name: column or list of column to have the type changed
        :type column_name: str | list
        :param column_type: type of the columns. See more details about these types at hopara.ColumnType.
        :type column_type: hopara.ColumnType
        """
        self.__update_column_properties(column_name, {'type': column_type.name})

    def set_unique_key_columns(self, column_name: Union[str, list]):
        """Set a column as unique, same value will update the same row instead of add a new one.
        :param column_name: column or list of column that must have unique values
        :type column_name: str | list
        """
        self.__update_column_properties(column_name, {'uniqueKey': True})

    def set_primary_key_column(self, column_name: Union[str, list]):
        """Set a column as primary key
        :param column_name: column or list of column that must be primary key
        :type column_name: str | list
        """
        self.__update_column_properties(column_name, {'primaryKey': True})

    def set_searchable_columns(self, column_name: Union[str, list]):
        """Set a column as searchable, which means an index will be created to speed up the queries
         :param column_name: column or list of column that must have an index created
         :type column_name: str | list
         """
        self.__update_column_properties(column_name, {'searchable': True})

    def disable_update_on_upsert(self, column_name: Union[str, list]):
        """When you need continually update the same row in a table, you can lose the information in the columns that you don't send on the new data.
        To avoid it, you can lock some columns and update the other without losing information.
        :param column_name: column or list of column that must have update disabled on upsert
        :type column_name: str | list
        """
        self.__update_column_properties(column_name, {'updateOnUpsert': False})



import json
import logging

from requests.compat import urljoin

from hopara.config import Config
from hopara.request import Request
from hopara.table import Table

BATCH_SIZE = 50000


class Hopara:
    """This class handles all data operations such as creating tables and inserting rows.
    """
    def __init__(self, organization: str = None):
        """Initialize with the organization you want to operate.
        Usually the organization is the domain of your work e-mail (e.g. mycompany.com).
        :param organization: Name of the organization.
        :type organization: str
        """
        self.config = Config()
        self.request = Request(self.config, {'tenant': organization} if organization else None)
        logging.info(f'ORGANIZATION: {organization}')
        logging.info(f'APP: {self.config.get_app_url()}')
        logging.info(f'HOST: {self.config.get_dataset_url()}')
        logging.info(f'USER: {self.config.get_client_id() or self.config.get_email()}')

    def get_table_url(self, table_name: str) -> str:
        return urljoin(self.config.get_dataset_url(), f'/table/{table_name}/')

    def get_row_url(self, table_name: str) -> str:
        return urljoin(self.get_table_url(table_name), 'row')

    def delete_table(self, table: Table):
        """ Delete a table
        :param table: table name
        :type table: hopara.Table
        """
        url = self.get_table_url(table.name)
        response = self.request.delete(url, table.get_payload())
        logging.info(f'RESPONSE: {response} / {response.content}\n')

    def create_table(self, table: Table, recreate: bool = False):
        """ Create a table
        :param table: table name
        :type table: hopara.Table
        :param recreate: If set to ``True`` the table will be deleted and recreated. If set to ``False`` new columns will be added to the existing table.
        Default: False.
        **If True is set all data previously store in the table will be permanently removed.**
        :type recreate: bool
        """
        if recreate:
            self.delete_table(table)
        url = self.get_table_url(table.name)
        logging.info(f'URL: {url}')
        logging.info(f'TABLE: {json.dumps(table.get_payload())}')
        response = self.request.post(url, table.get_payload())
        logging.info(f'RESPONSE: {response} / {response.content}\n')

    def __insert_rows(self, url, rows: list):
        response = self.request.post(url, rows)
        logging.info(f'RESPONSE: {response} / {response.content}')

    def insert_rows(self, table: Table, rows: list):
        """ Insert rows in a table.
        :param table: table object
        :type table: hopara.Table
        :param rows: the data to be inserted in the following format: ``[{'col1': 1, 'col2': 'a'}, {'col1': 2, 'col2': 'b'}]``
        """
        url = self.get_row_url(table.name)
        logging.info(f'URL: {url}')
        logging.info(f'SAMPLE: {json.dumps(rows[0])}')

        if BATCH_SIZE >= len(rows):
            return self.__insert_rows(url, rows)
        batches = range(0, len(rows), BATCH_SIZE)
        logging.info(f'Processing {len(rows):,} rows in {len(batches):,} batches of {BATCH_SIZE:,} rows each...')
        for i, start in enumerate(batches, 1):
            end = min(start + BATCH_SIZE, len(rows))
            self.__insert_rows(url, rows[start:end])

    def update_rows(self, table: Table, new_values: dict, filters: list):
        """Updated rows. The filters parameter can be used to restrict the affected rows.
        :param table: table name
        :type table: hopara.Table
        :param new_values: the new values for the columns in the following format:
        ``{'column_name1': new_value, 'column_name2': new_value2}``
        :type new_values: dict
        :param filters: a list of filter operations, you must use the hopara.Filter to generate it:
        ``[Filter('column_name1') > 10, Filter('column_name2') == 'name', Filter('column_name3') == False]``
        You can use as many Filters as you want to select the rows.
        :type filters: list
        """
        url = urljoin(self.get_table_url(table.name), 'update')
        response = self.request.post(url, {"values": new_values, "filters": filters})
        logging.info(f'RESPONSE: {response} / {response.content}\n')


if __name__ == "__main__":
    hopara = Hopara()



from enum import Enum, auto


class ColumnType(Enum):
    """hopara.Table support all the following types for its columns:
     - ``STRING``, ``INTEGER``, ``DECIMAL``, ``BOOLEAN``
     - ``DATETIME``: python datetime format
     - ``AUTO_INCREMENT``: auto-increment integer
     - ``MONEY``: currency values
     - ``JSON``: json column, can be used for regular Json or a (GeoJson)(https://geojson.org/)
     - ``STRING_ARRAY``: string array column: ``['value1', 'value2']``
     - ``POLYGON``: a string representing a Polygon ``POLYGON ((40 40, 20 45, 45 30, 40 40))`` in [WKT format](https://en.wikipedia.org/wiki/Well-known_text_representation_of_geometry)
     - ``MULTIPOINT``: a string representing a MultiPoint ``MULTIPOINT ((10 40), (40 30), (20 20), (30 10))`` in WKT format
     - ``IMAGE``: a string that representing an image ``data:[<mediatype>][;base64],<data>`` in [Data URIs format](https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/Data_URIs)
    """
    STRING = auto()
    INTEGER = auto()
    DECIMAL = auto()
    BOOLEAN = auto()
    DATETIME = auto()
    AUTO_INCREMENT = auto()
    MONEY = auto()
    JSON = auto()
    STRING_ARRAY = auto()
    POLYGON = auto()
    MULTIPOINT = auto()
    IMAGE = auto()
